package com.hive.udf;

import org.apache.hadoop.hive.ql.metadata.HiveException;
import org.apache.hadoop.io.Text;
import java.nio.charset.StandardCharsets;
import org.apache.hadoop.hive.ql.exec.UDFArgumentException;
import org.apache.hadoop.hive.serde2.objectinspector.primitive.PrimitiveObjectInspectorFactory;
import org.apache.hadoop.hive.ql.exec.UDFArgumentTypeException;
import org.apache.hadoop.hive.ql.exec.UDFArgumentLengthException;
import org.apache.hadoop.hive.serde2.objectinspector.ObjectInspector;
import java.io.IOException;
import java.io.File;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.ArrayList;
import java.util.Map;
import java.util.List;
import org.apache.hadoop.hive.ql.udf.generic.GenericUDF;

public class IpToLocUdf extends GenericUDF
{
    public static List<Map<String, String>> mapList;
    
    static {
        IpToLocUdf.mapList = new ArrayList<Map<String, String>>();
        final String filePath = "/root/eduhq/data/area.json";
        try {
            final ObjectMapper objectMapper = new ObjectMapper();
            IpToLocUdf.mapList = objectMapper.readValue(new File(filePath), List.class);
        }
        catch (final IOException e) {
            e.printStackTrace();
        }
    }
    
    public ObjectInspector initialize(final ObjectInspector[] objectInspectors) throws UDFArgumentException {
        if (objectInspectors.length != 1) {
            throw new UDFArgumentLengthException("传入的参数长度不正确!");
        }
        if (!objectInspectors[0].getCategory().equals((Object)ObjectInspector.Category.PRIMITIVE)) {
            throw new UDFArgumentTypeException(0, "传入的参数类型不正确!!!");
        }
        return (ObjectInspector)PrimitiveObjectInspectorFactory.javaStringObjectInspector;
    }
    
    public Object evaluate(final GenericUDF.DeferredObject[] deferredObjects) throws HiveException {
        if (deferredObjects[0].get() == null) {
            return "";
        }
        final int index = (int)(Math.random() * IpToLocUdf.mapList.size());
        final String pro_city = String.valueOf(IpToLocUdf.mapList.get(index).get("province")) + "_" + (String)IpToLocUdf.mapList.get(index).get("city");
        final Text new_str = new Text(pro_city.getBytes(StandardCharsets.UTF_8));
        return new_str;
    }
    
    public String getDisplayString(final String[] strings) {
        return "";
    }
    
    public static void main(final String[] args) {
        final int index = (int)(Math.random() * IpToLocUdf.mapList.size());
        final String pro_city = String.valueOf(IpToLocUdf.mapList.get(index).get("province")) + "_" + (String)IpToLocUdf.mapList.get(index).get("city");
        System.out.println(pro_city);
    }
}
